@@ -938,7 +938,7 @@ fhandler_socket_unix::open_pipe (PUNICODE_STRING pipe_name, bool xchg_sock_info)
   status = NtOpenFile (&ph, access, &attr, &io, sharing, 0);
   if (NT_SUCCESS (status))
     {
-      set_io_handle (ph);
+      set_handle (ph);
       if (xchg_sock_info)
 	send_sock_info (false);
     }
@@ -1149,6 +1149,16 @@ fhandler_socket_unix::set_cred ()
   scred->gid = myself->gid;
 }
 
+void
+fhandler_socket_unix::fixup_helper ()
+{
+  if (shmem_handle)
+    reopen_shmem ();
+  connect_wait_thr = NULL;
+  cwt_termination_evt = NULL;
+  cwt_param = NULL;
+}
+
 /* ========================== public methods ========================= */
 
 void
@@ -1158,20 +1168,15 @@ fhandler_socket_unix::fixup_after_fork (HANDLE parent)
   if (backing_file_handle && backing_file_handle != INVALID_HANDLE_VALUE)
     fork_fixup (parent, backing_file_handle, "backing_file_handle");
   if (shmem_handle)
-    {
-      fork_fixup (parent, shmem_handle, "shmem_handle");
-      reopen_shmem ();
-    }
-  connect_wait_thr = NULL;
-  cwt_termination_evt = NULL;
-  cwt_param = NULL;
+    fork_fixup (parent, shmem_handle, "shmem_handle");
+  fixup_helper ();
 }
 
 void
 fhandler_socket_unix::fixup_after_exec ()
 {
   if (!close_on_exec ())
-    fixup_after_fork (NULL);
+    fixup_helper ();
 }
 
 void
@@ -1201,12 +1206,6 @@ fhandler_socket_unix::dup (fhandler_base *child, int flags)
       return -1;
     }
   fhandler_socket_unix *fhs = (fhandler_socket_unix *) child;
-  if (reopen_shmem () < 0)
-    {
-      __seterrno ();
-      fhs->close ();
-      return -1;
-    }
   if (backing_file_handle && backing_file_handle != INVALID_HANDLE_VALUE
       && !DuplicateHandle (GetCurrentProcess (), backing_file_handle,
 			    GetCurrentProcess (), &fhs->backing_file_handle,
@@ -1224,6 +1223,12 @@ fhandler_socket_unix::dup (fhandler_base *child, int flags)
       fhs->close ();
       return -1;
     }
+  if (fhs->reopen_shmem () < 0)
+    {
+      __seterrno ();
+      fhs->close ();
+      return -1;
+    }
   fhs->sun_path (sun_path ());
   fhs->peer_sun_path (peer_sun_path ());
   fhs->connect_wait_thr = NULL;
@@ -1365,7 +1370,7 @@ fhandler_socket_unix::socket (int af, int type, int protocol, int flags)
   if (flags & SOCK_CLOEXEC)
     set_close_on_exec (true);
   init_cred ();
-  set_io_handle (NULL);
+  set_handle (NULL);
   set_unique_id ();
   set_ino (get_unique_id ());
   return 0;
@@ -1412,7 +1417,7 @@ fhandler_socket_unix::socketpair (int af, int type, int protocol, int flags,
   pipe = create_pipe (true);
   if (!pipe)
     goto create_pipe_failed;
-  set_io_handle (pipe);
+  set_handle (pipe);
   sun_path (&sun);
   fh->peer_sun_path (&sun);
   connect_state (listener);
@@ -1436,10 +1441,10 @@ fhandler_socket_unix::socketpair (int af, int type, int protocol, int flags,
 fh_open_pipe_failed:
   NtClose (pipe);
 create_pipe_failed:
-  NtUnmapViewOfSection (GetCurrentProcess (), fh->shmem);
+  NtUnmapViewOfSection (NtCurrentProcess (), fh->shmem);
   NtClose (fh->shmem_handle);
 fh_shmem_failed:
-  NtUnmapViewOfSection (GetCurrentProcess (), shmem);
+  NtUnmapViewOfSection (NtCurrentProcess (), shmem);
   NtClose (shmem_handle);
   return -1;
 }
@@ -1483,12 +1488,12 @@ fhandler_socket_unix::bind (const struct sockaddr *name, int namelen)
 	  binding_state (unbound);
 	  return -1;
 	}
-      set_io_handle (pipe);
+      set_handle (pipe);
     }
   backing_file_handle = unnamed ? autobind (&sun) : create_file (&sun);
   if (!backing_file_handle)
     {
-      set_io_handle (NULL);
+      set_handle (NULL);
       if (pipe)
 	NtClose (pipe);
       binding_state (unbound);
@@ -1538,7 +1543,7 @@ fhandler_socket_unix::listen (int backlog)
       connect_state (unconnected);
       return -1;
     }
-  set_io_handle (pipe);
+  set_handle (pipe);
   state_lock ();
   set_cred ();
   state_unlock ();
@@ -1575,7 +1580,7 @@ fhandler_socket_unix::accept4 (struct sockaddr *peer, int *len, int flags)
       else
 	{
 	  /* Set new io handle. */
-	  set_io_handle (new_inst);
+	  set_handle (new_inst);
 	  io_unlock ();
 	  /* Prepare new file descriptor. */
 	  cygheap_fdnew fd;
@@ -1600,7 +1605,7 @@ fhandler_socket_unix::accept4 (struct sockaddr *peer, int *len, int flags)
 		  sock->pc.set_nt_native_path (pc.get_nt_native_path ());
 		  sock->connect_state (connected);
 		  sock->binding_state (binding_state ());
-		  sock->set_io_handle (accepted);
+		  sock->set_handle (accepted);
 
 		  sock->sun_path (sun_path ());
 		  sock->sock_cred (sock_cred ());
@@ -1814,7 +1819,7 @@ fhandler_socket_unix::close ()
     NtClose (shm);
   param = InterlockedExchangePointer ((PVOID *) &shmem, NULL);
   if (param)
-    NtUnmapViewOfSection (GetCurrentProcess (), param);
+    NtUnmapViewOfSection (NtCurrentProcess (), param);
   return 0;
 }
 
